<?php

    if ( !defined('K_COUCH_DIR') ) die(); // cannot be loaded directly
    require_once( K_COUCH_DIR.'addons/member/member.php' );
    
    class KFacebook{
        var $t = array();
        
        function KFacebook(){
            global $FUNCS;
            
            // config
            $this->populate_config();
            
            // register tags
            $FUNCS->register_tag( 'facebook_login', array('KFacebook', 'facebook_login_handler'), 1 );
            $FUNCS->register_tag( 'facebook_login_link', array('KFacebook', 'facebook_login_link_handler') );
            
        }
        
        function populate_config(){
            
            $t = array();
            if( file_exists(K_COUCH_DIR.'addons/social-login/facebook/config.php') ){
                require_once( K_COUCH_DIR.'addons/social-login/facebook/config.php' );
            }
            else{
                die( 
                      "<h3>social-login/facebook module: 'config.php' not found. <br/>
                      Perhaps you forgot to rename the 'config.example.php' file in 'couch/addons/social-login/facebook' to 'config.php'?
                      </h3>" 
                   );
            }
            $this->t = array_map( "trim", $t );
            unset( $t );
        }
        
        function process_login( $node ){
            global $DB, $FUNCS, $CTX, $KMEMBER;
            
            require_once( K_COUCH_DIR.'addons/social-login/facebook/sdk/facebook.php' );
            
            $facebook = new Facebook( array(
                'appId' => $this->t['app_id'],
                'secret' => $this->t['app_secret'],
                'fileUpload' => false, 
                'allowSignedRequest' => false
            ) );
            
            
            if( isset($_REQUEST['state']) ){
                // being called back with the authorization code from Facebook
                
                
                $user_id = $facebook->getUser();
                if( !$user_id ){
                    return KFuncs::raise_error( $this->t['error'] );
                }
                
                // get email
                try{
                    $user_profile = $facebook->api( '/me','GET' );
                    if( !isset($user_profile['email']) || empty($user_profile['email']) ){
                        return KFuncs::raise_error( $this->t['email_error'] );
                    }
                    
                }catch( FacebookApiException $e ){
                    return KFuncs::raise_error( $this->t['error'] );
                }
                
                // at this point we have the visitor's email
                // Check if an account already exists on our system with this email
                $user = $KMEMBER->get_user( $user_profile['email'] );
                if( is_null($user) ){
                    
                    // create one with the same email
                    $params = array();
                    $params[] = array('lhs'=>'_masterpage', 'op'=>'=', 'rhs'=>$KMEMBER->members_tpl);
                    $params[] = array('lhs'=>'_mode', 'op'=>'=', 'rhs'=>'create');
                    $params[] = array('lhs'=>'k_page_title', 'op'=>'=', 'rhs'=>$user_profile['name']);
                    $params[] = array('lhs'=>'k_page_name', 'op'=>'=', 'rhs'=>md5($KMEMBER->hasher->get_random_bytes(16)));
                    $params[] = array('lhs'=>'member_email', 'op'=>'=', 'rhs'=>$user_profile['email']);
                    $password = $FUNCS->generate_key( 12 );
                    $params[] = array('lhs'=>'member_password', 'op'=>'=', 'rhs'=>$password);
                    $params[] = array('lhs'=>'member_password_repeat', 'op'=>'=', 'rhs'=>$password);
                    $params[] = array('lhs'=>'member_active', 'op'=>'=', 'rhs'=>1);
                    
                    // delegate to cms:db_persist
                    require_once( K_COUCH_DIR.'addons/data-bound-form/data-bound-form.php' );
                    $node->name='db_persist';
                    KDataBoundForm::db_persist( $params, $node );
                    
                    // succesfully created?
                    if( $CTX->get('k_error' )){
                        return KFuncs::raise_error( $CTX->get('k_error' ) );
                    }
                    
                    $user = $KMEMBER->get_user( $user_profile['email'] );
                    if( is_null($user) ){ // shouldn't happen
                        return KFuncs::raise_error( $this->t['error'] );
                    }
                }
                
                // login the user
                $KMEMBER->set_cookie( (string)$user->member_id, 0 );
                
                // redirect
                // Facebook appends '#_=_' to the redirection URL :(
                // https://developers.facebook.com/x/bugs/318390728250352/
                $url = $FUNCS->unhtmlentities( $_GET['redirect'], K_CHARSET );
                $KMEMBER->redirect( $url );
                
            }
            else{
                // redirect the visitor to Facebook for login
                $url = $facebook->getLoginUrl( array(
                    'scope' => 'email'
                ) );
                
                $DB->commit( 1 ); // force commit, we are redirecting.
                header( "Location: ".$url );
            }
            
        
        }
        
        // Tag handlers
        function facebook_login_handler( $params, $node ){
            global $FUNCS, $CTX, $KFACEBOOK, $KMEMBER, $PAGE;
            
            $PAGE->no_cache = 1;
            
            $res = $KFACEBOOK->process_login( $node );
            
            if( $FUNCS->is_error($res) ){
                $CTX->set( 'k_error', $res->err_msg, 'global' );
            }
        }
        
        function facebook_login_link_handler( $params, $node ){
            global $FUNCS, $CTX, $KFACEBOOK;
            
            if( $CTX->get('k_member_logged_in') ){
                $link = 'javascript:void(0)';
            }
            else{
                if( K_PRETTY_URLS ){
                    $link = K_SITE_URL . $FUNCS->get_pretty_template_link( $KFACEBOOK->t['login_tpl'] );
                }
                else{
                    $link = K_SITE_URL . $KFACEBOOK->t['login_tpl'];
                }
                $redirect =  trim( $_GET['redirect'] );
                if( !strlen($redirect) ){ $redirect = $_SERVER["REQUEST_URI"]; }
                
                $link .= '?redirect=' . urlencode( $redirect );
            }
            
            return $link;
            
        }
    }
    
    $KFACEBOOK = new KFacebook();
